package com.ruoyi.project.business.controller.contract;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.business.domain.TqContractHistory;
import com.ruoyi.project.business.service.ITqContractHistoryService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 合同履历Controller
 * 
 * @author ruoyi
 * @date 2024-08-16
 */
@RestController
@RequestMapping("/business/contractHistory")
public class TqContractHistoryController extends BaseController
{
    @Resource
    private ITqContractHistoryService tqContractHistoryService;

    /**
     * 查询合同履历列表
     */
    // @PreAuthorize("@ss.hasPermi('system:history:list')")
    @GetMapping("/list")
    public TableDataInfo list(TqContractHistory tqContractHistory)
    {
        startPage();
        List<TqContractHistory> list = tqContractHistoryService.selectTqContractHistoryList(tqContractHistory);
        return getDataTable(list);
    }

    /**
     * 导出合同履历列表
     */
    // @PreAuthorize("@ss.hasPermi('system:history:export')")
    @Log(title = "合同履历", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TqContractHistory tqContractHistory)
    {
        List<TqContractHistory> list = tqContractHistoryService.selectTqContractHistoryList(tqContractHistory);
        ExcelUtil<TqContractHistory> util = new ExcelUtil<TqContractHistory>(TqContractHistory.class);
        util.exportExcel(response, list, "合同履历数据");
    }

    /**
     * 获取合同履历详细信息
     */
    // @PreAuthorize("@ss.hasPermi('system:history:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tqContractHistoryService.selectTqContractHistoryById(id));
    }

    /**
     * 获取合同履历详细信息
     */
    // @PreAuthorize("@ss.hasPermi('system:history:query')")
    @GetMapping(value = "/getContractHistory4Contract/{id}")
    public AjaxResult getContractHistory4Contract(@PathVariable("id") Long id)
    {
        return success(tqContractHistoryService.getContractHistory4Contract(id));
    }

    /**
     * 新增合同履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:add')")
    @Log(title = "合同履历", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TqContractHistory tqContractHistory)
    {

        return toAjax(tqContractHistoryService.insertTqContractHistory(tqContractHistory));
    }

    /**
     * 修改合同履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:edit')")
    @Log(title = "合同履历", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TqContractHistory tqContractHistory)
    {
        return toAjax(tqContractHistoryService.updateTqContractHistory(tqContractHistory));
    }

    /**
     * 删除合同履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:remove')")
    @Log(title = "合同履历", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tqContractHistoryService.deleteTqContractHistoryByIds(ids));
    }
}
